/*
   SPDX-FileCopyrightText: 2016-2026 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once
#include <QObject>

class FilterActionMissingAccountDialogTest : public QObject
{
    Q_OBJECT
public:
    explicit FilterActionMissingAccountDialogTest(QObject *parent = nullptr);
    ~FilterActionMissingAccountDialogTest() override;
private Q_SLOTS:
    void shouldHaveDefaultValue();
};
